@@ -419,6 +419,17 @@ class BaseInstance(DataMixin, models.Model):
     def get_play_url(self):
         return ""
 
+    def finish_character(self, character: BaseCharacterData):
+        assert character.instance_id == self.id
+        character.finish()
+
+        characters = self.character_set.exclude(user=None)
+        total = characters.count()
+        finished = characters.exclude(_date_finished=None).count()
+        if total and total == finished:
+            self.date_end = timezone.now()
+            self.save()
+
 
 class Instance(BaseInstance):
     """
@@ -432,6 +443,7 @@ class Instance(BaseInstance):
 class BaseCharacterData(DataMixin, models.Model):
     name = models.CharField("In-game name", max_length=200)
     data: dict = JSONField(editable=False, default=dict, blank=True)
+    _date_finished = models.DateTimeField(null=True, blank=True)
 
     class Meta:
         abstract = True
@@ -440,6 +452,10 @@ class BaseCharacterData(DataMixin, models.Model):
     def __str__(self):
         return self.name
 
+    @property
+    def date_finished(self):
+        return self._date_finished
+
     def save(self, *args, **kwargs):
         if not self.name:
             self.name = self.get_default_name()
@@ -455,6 +471,10 @@ class BaseCharacterData(DataMixin, models.Model):
             existing_names=other_characters.values_list("name", flat=True)
         )
 
+    def finish(self):
+        self._date_finished = timezone.now()
+        self.save()
+
 
 class BaseCharacterLinked(models.Model):
     instance: Instance = models.ForeignKey(
